// A MouseJiggler for the DigiSpark attiny85 Board.
// This Board is rather limited but for only jiggling its better than a
// Leonardo or Micro (or Clone) as it only appears as HID Mouse and nothing more
// (After the 5-6 Seconds for the BootLoader)
//
// If you encouter that the jiggler does not appear as a HID Mouse after 5 Seconds
// you may need to upgrade the bootloader first.
// The latest is to be found at https://github.com/micronucleus/micronucleus/raw/master/upgrade/releases/upgrade-t85_default.hex
// Update with the tool micronucleus:
// <unplug stick><plug stick>./micronucleus --run upgrade-t85_default.hex
// This takes a while so please be patient (Should be up to 30 Seconds).
// After the upgrade delete the upgrader please with
// <unplug stick><plug stick>./micronucleus --erase-only
// Keep in Mind: you only have at most 5 Seconds from <plug stick> to ./micronucleus ...

// Sketch uses 4876 bytes (81%) of program storage space. Maximum is 6012 bytes.
// Global variables use 133 bytes of dynamic memory.

#include <DigiMouse.h>

#define PIXEL 3          // The maximum Move the jiggler does
#define MINWAIT 5000     // Wait minimum between two jiggles in ms
#define MAXWAIT 15000    // maximum between two jiggles

#define LEDPIN 1         // Onboard on most recent attiny85 boards (digispark compatible)
#define BUTTON 0         // Short to gnd with a button to toggle the jiggler state
// If jiggler is disabled the LED Heartbeat is inverted
#define BUTTONLONG 5000  // Pressing the Button for this time in ms will reset to bootloader
// Keep pressed until micronucleus flash starts

void setup() {
  randomSeed(analogRead(0));   // Initialize with some random noise from one analog input
  pinMode(LEDPIN, OUTPUT);
  pinMode(BUTTON, INPUT_PULLUP);
  DigiMouse.begin();
}

byte jiggler_enabled = 1;   // 0 = jiggling disabled, 1= jiggling enabled. Toggled via Button on Pin "BUTTON"

// Jiggles the Mouse when it's time to do so
// For that we compare the actual time (millis())
// and the time we last did something (lastrun)
// if the difference is more than wait (which varies between the steps)
// then do something
void jiggle() {
  static int state = 0;
  static int x, y;
  static long int wait = 0;
  static unsigned long lastrun = 0;
  unsigned long int nowms;
  nowms = millis();
  if (nowms - lastrun >= wait) {  // Next runtime reached?
    lastrun = nowms;
    switch (state) {
      case 0:
        if (jiggler_enabled == 1) {   // We only check this variable in this state (step) so that turning off
          x = random(-PIXEL, PIXEL);  // moves back the cursor to the old position first
          y = random(-PIXEL, PIXEL);  // We are using random() to pseudo-randomly move the cursor
          DigiMouse.move(x, y, 0);
          wait = random(MINWAIT / 100, MAXWAIT / 100);
          state = 1;
        } else {
          wait = 500;
        }
        break;
      default:
        // move backwards;
        DigiMouse.move(0 - x, 0 - y, 0);
        wait = random(MINWAIT, MAXWAIT);
        state = 0;
        break;
    }
  }
}

// Implementing a Heartbeat LED
// Scheduler works the same as in the jiggle()
void led() {
  static int state;
  static int ledvalue;
  static long int wait = 0;
  static unsigned long lastrun = 0;
  unsigned long int nowms;
  nowms = millis();
  if (nowms - lastrun >= wait) {  // Next runtime reached?
    lastrun = nowms;
    switch (state) {
      case 0:
      case 2:
        ledvalue += 24;
        if (ledvalue >= 255) {
          ledvalue = 255;
          state++;
        }
        wait = 15;
        break;
      case 1:
      case 3:
        ledvalue -= 24;
        if (ledvalue <= 0) {
          ledvalue = 0;
          state++;
        }
        wait = 15;
        break;
      default:
        wait = 500;
        state = 0;
        break;
    }
    if (jiggler_enabled == 0) { // Invert output if jiggler is disabled
      analogWrite(LEDPIN, 255 - ledvalue);
    } else {
      analogWrite(LEDPIN, ledvalue); // We are using Hardware PWM Output for the LED
    }
  }                                // So the Heartbeat is somewhat smooth
}

void reboot(void) {  // This code is from https://digistump.com/wiki/digispark/tricks
  noInterrupts(); // disable interrupts which could mess with changing prescaler
  CLKPR = 0b10000000; // enable prescaler speed change
  CLKPR = 0; // set prescaler to default (16mhz) mode required by bootloader
  void (*ptrToFunction)(); // allocate a function pointer
  ptrToFunction = 0x0000; // set function pointer to bootloader reset vector
  (*ptrToFunction)(); // jump to reset, which bounces in to bootloader
}

void button () {
  static unsigned long lasthigh = 0;
  static unsigned long lastedge = 0;
  unsigned long nowms;
  int state;
  static int oldstate = LOW;
  state = digitalRead(BUTTON);
  nowms = millis();
  if (state == LOW && oldstate == HIGH) {  // EDGE detected
    if ((nowms - lastedge) > 200) {        // But only if no Edge the last 200ms
      if (jiggler_enabled == 1) {
        jiggler_enabled = 0;
      } else {
        jiggler_enabled = 1;
      }
    }
    lastedge = nowms;
  }
  if (state == LOW) {
    if (lasthigh != 0) {                      // Only trigger if the pin was HIGH at least once
      if (nowms - lasthigh > BUTTONLONG) {    // State needs to be stable the last BUTTONLONG ms
        digitalWrite(LEDPIN, LOW);
        delay(150);
        for (state = 0; state < 5; state++) { // Blink 5 times to sigmal now is reset-time
          digitalWrite(LEDPIN, HIGH);         // We are re-using the variable state for the loop
          delay(150);                         // You need to keep the button pressed until reflash is
          digitalWrite(LEDPIN, LOW);          // starting if you have a ENTRY_JUMPER bootloader installed!
          delay(150);
        }
        reboot();                             // Reboot to bootloader
        lasthigh = 0;                         // Unnecessary as this code should never be reached
      }
    }
  }
  oldstate = state;
  if (state != LOW) {
    lasthigh = nowms;
  }
}

// Code from DigiMouse.h but patched out the "noisyness" causing the mouse jiggling all the time without a move
void usbupdate() {
  usbPoll();
  // if the report has changed, try force an update anyway
  if (memcmp(last_built_report, last_sent_report, REPORT_SIZE)) {
    must_report = 1;
  }
  // if we want to send a report, signal the host computer to ask us for it with a usb 'interrupt'
  if (must_report) {
    if (usbInterruptIsReady()) {
      must_report = 0;
      buildReport(reportBuffer); // put data into reportBuffer
      clearMove(); // clear deltas
      usbSetInterrupt(reportBuffer, REPORT_SIZE);
    }
  }
}

void loop() {
  button();                               // Check the Button
  jiggle();                               // Jiggle the mouse
  led();                                  // Heartbeat LED
  usbupdate();                            // Manually call usbupdate() - a patched Version of DigiMouse.update() which causes unwanted "noise"
  delay(5);
}
